/* See license.txt for terms of usage */

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
Components.utils.import("chrome://firetv-comp/content/http-channel.jsm");

var EXPORTED_SYMBOLS = [ "TvChannel", "scope", "FBTrace" ];

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

const CAT = "[tv://] ";

// ------------- TvChannel --
function TvChannel(aUri, aReferrer, notifyObservers) {
	if (FBTrace.DBG_FIRETV_TVCHANNEL) {
		FBTrace.sysout(CAT + "[TvChannel] [constructor]");
	}
	HttpChannel.apply(this, arguments);
}

TvChannel.prototype = new HttpChannel();

(function(proto) {

	// Override BaseChannel.openContentStream
	Object.defineProperties(proto, {
		openContentStream : {
			value : function() {
				if (FBTrace.DBG_FIRETV_TVCHANNEL) {
					FBTrace.sysout(CAT + "[TvChannel.openContentStream]", scope.FireTVPlugin);
				}

				var tabId = null, aBrowser = null, host;
				try {
					aBrowser = scope.FireTVPlugin.utils.getBrowserFromChannel(this);
					var aTab = scope.FireTVPlugin.TabManager.getTabForBrowser(aBrowser);
					if(!aTab){
						// -- if we don't have the tab, do the best effort by taking the currently active tab
						// This may happen, when Firebug network panel try to do its own requests (i.e to display an image)
						aTab = scope.document.getElementById("content").selectedTab;
					}
					if (aBrowser == null){
						var tabbedBrowser = aTab.ownerDocument.defaultView.gBrowser;
						aBrowser = tabbedBrowser.getBrowserForTab(aTab);
					}
					tabId = aTab.getAttribute("firetv-tab-id");
					if (FBTrace.DBG_FIRETV_TVCHANNEL) {
						FBTrace.sysout(CAT + "[TvChannel.openContentStream] tab-id: " + tabId);
					}
				} catch (e) {
					if (FBTrace.DBG_FIRETV_TVCHANNEL || FBTrace.DBG_FIRETV_ERROR) {
						FBTrace.sysout(CAT + "[TvChannel.openContentStream] ERROR while getting originating tab!", {error:e, channel:this});
					}
				} finally {
					if (!tabId || tabId == ""){
						// -- not firetv enabled tab
						if (FBTrace.DBG_FIRETV_TVCHANNEL || FBTrace.DBG_FIRETV_ERROR) {
							FBTrace.sysout(CAT + "[TvChannel.openContentStream] ERROR while getting originating tab!");
						}
						throw "Not a firetv enabled tab!";
					}
				}
					
				var infos = scope.FireTVPlugin.TabManager.infos[tabId];
				var profile = infos.profile;
				host = (aBrowser.currentURI.scheme== "dvb")?"dvb:/":(aBrowser.currentURI.scheme + "://" + aBrowser.currentURI.hostPort);
				
				var params = {};
				var path = this.URI.path;
				var queryString, keyValues, keyValue, index, i, value;
				index = path.indexOf("?");
				if (index > -1) {
					queryString = path.substring(index + 1);
					keyValues = queryString.split("&");
					for (i = 0; i < keyValues.length; i++) {
						keyValue = keyValues[i].split("=");
						value = keyValue[1];
						if (value) {
							value = scope.decodeURIComponent(value);
						}
						params[keyValue[0]] = value;
					}
				}

				var channelList = (scope.FireTVPlugin.PreferenceManager.customChannelListSelected())?scope.FireTVPlugin.Broadcast.customChannelList:scope.FireTVPlugin.Broadcast.defaultChannelList;
				var channel = scope.FireTVPlugin.Broadcast.getChannelByCcid(scope.FireTVPlugin.PreferenceManager
						.getHostRelatedPref(host, "tv-channel"));
				if (channel == null) {
					channel = channelList[0];
				}
				var channelDisplayName = channel.name;
				params.channelDisplayName = channelDisplayName;
				if (FBTrace.DBG_FIRETV_TVCHANNEL) {
					FBTrace.sysout(CAT + "[TvChannel.writeTvImage] options", params);
				}
				var svgImage = scope.FireTVPlugin.SVG.getTVImage(profile, host, params);

				this.contentCharset = "utf-8";
				this.contentType = "image/svg+xml";
				this.contentLength = svgImage.length;

				var is = Cc["@mozilla.org/io/string-input-stream;1"].createInstance(Ci.nsIStringInputStream);
				is.data = svgImage;
				HttpChannel.prototype.openContentStream.apply(this, arguments);
				return is;
				
			},
			enumerable : false
		}
	});
})(TvChannel.prototype);
