/* See license.txt for terms of usage */

try {
	(function() {
		if (this.Bridge) {
			return;
		}

		const Cc = Components.classes;
		const Ci = Components.interfaces;
		const Cr = Components.results;
		
		var _this = this;
		
		var FBTrace = this.FBTrace;

		var bridge = {};

		bridge.CAT = "[ftv.bridge] ";

		var observerService = Cc["@mozilla.org/observer-service;1"]
				.getService(Ci.nsIObserverService);

		var Delegate = function() {
		};

		var bridgeInterface = {
			getProfiles : function(win) {
				var res = [], i, profileName, profileDefinition;
				for (i = 0; i < _this.ProfileManager.profilesNames.length; i++) {
					profileName = _this.ProfileManager.profilesNames[i];
					profileDefinition = _this.ProfileManager.getProfileDefinition(profileName);
					res.push({
						value : profileName,
						label : profileDefinition.label,
						icon : profileDefinition.icon
					});
				}
				return res;
			},
			getCurrentProfile : function(win) {
				var infos = _this.TabManager.getTabInfosForWindow(win);
				var currentProfileName = infos.profile;
				var profileDefinition = _this.ProfileManager.getProfileDefinition(currentProfileName);
				return {
					value : currentProfileName,
					label : profileDefinition.label,
					icon : profileDefinition.icon
				};
			},

			getTVFormats : function(win) {
				var res = [ {
					value : "RAW",
					label : "RAW"
				}, {
					value : "4:3",
					label : "4:3"
				}, {
					value : "3:2",
					label : "3:2"
				}, {
					value : "16:9",
					label : "16:9"
				} ];
				return res;
			},
			getCurrentTVFormat : function(win) {
				var val = win.document.documentElement.getAttribute("firetv-tv-format");
				if (!val) {
					var infos = _this.TabManager.getTabInfosForWindow(win);
					var currentProfileName = infos.profile;
					var profileDefinition = _this.ProfileManager.getProfileDefinition(currentProfileName);
					if (profileDefinition.defaultTVFormat) {
						val = profileDefinition.defaultTVFormat;
					} else {
						val = "RAW";
					}
				}
				return {
					value : val,
					label : val
				};
			},

			getMarginDisplay : function(win) {
				var val = win.document.documentElement.getAttribute("firetv-margin-display");
				var host;
				if (!val) {
					host = window.location.protocol + "//" + window.location.host;
					val = _this.PreferenceManager.getHostRelatedPref(host, "margin-display");
					if (!val) {
						val = _this.PreferenceManager.getGlobalMarginDisplay();
					}
				}
				return val;
			},
			
			getMediaPlayers : function(win) {
				var res = [ {
					value : "SVGPlayer",
					label : "SVG Player"
				}, {
					value : "FlashPlayer",
					label : "Flash Player"
				}];
				return res;
			},
			
			getMediaPlayer : function(win) {
				var val = win.document.documentElement.getAttribute("firetv-media-player");
				var host;
				if (!val) {
					host = window.location.protocol + "//" + window.location.host;
					val = _this.PreferenceManager.getHostRelatedPref(host, "media-player");
					if (!val) {
						val = _this.PreferenceManager.getDefaultMediaPlayer();
					}
				}
				var i;
				var impls = this.getMediaPlayers();
				for (i = 0; i<impls.length; i++){
					if (impls[i].value == val){
						return impls[i];
					}
				}
				return null;
			},

			getTVImageDisplay : function(win) {
				var val = win.document.documentElement.getAttribute("firetv-tvimage-display");
				var host;
				if (!val) {
					host = window.location.protocol + "//" + window.location.host;
					val = _this.PreferenceManager.getHostRelatedPref(host, "tvimage-display");
					if (!val) {
						val = false;
					}
				}
				return val;
			},

			chooseTVImage : function(win) {
				if (FBTrace.DBG_FIRETV_BRIDGE) {
					FBTrace.sysout(bridge.CAT + "[bridgeHandler.chooseTVImage]");
				}
				var tvImageFile = _this.FileUtils.chooseLocalFile(_this.Messages.getMessage("firetv.tvImage.fileChooserTitle"), Ci.nsIFilePicker.filterImages);
				if (FBTrace.DBG_FIRETV_BRIDGE) {
					FBTrace.sysout(bridge.CAT + "[bridgeHandler.chooseTVImage] file: " + tvImageFile);
				}
				if (tvImageFile !== null) {
					var success = _this.FileUtils.deleteFile(_this.FileUtils.getUserImageFile());
					success = success
							&& _this.FileUtils.copyFile(tvImageFile.path,
									_this.FileUtils.getUserStorageDirectory().path, "tv");

					if (success) {
						var gBrowser = document.getElementById("content");
						var doc = gBrowser.selectedBrowser.contentWindow.document.documentElement;
						doc.setAttribute("firetv-tvimage-display", "true");
						gBrowser.selectedBrowser.contentWindow.location.reload(true);
					} else {
						window.alert(_this.Messages.getMessage("firetv.tvImage.unableToCopy"));
					}
				}
			}
		};

		var bridgeHandler = function(event) {
			var win = event.target;
			var details = event.detail;
			var method = details.method;
			var callback = details.callback;
			var callbackArgs = (details.callbackArgs) ? details.callbackArgs : [];
			details.args = (details.args) ? details.args : [];
			details.args.splice(0, 0, win);
			var args = details.args;

			if (bridgeInterface[method]) {
				try {
					if (FBTrace.DBG_FIRETV_BRIDGE) {
						FBTrace
								.sysout(bridge.CAT + "[bridgeHandler] Calling method : " + method + "(" + args.join("," + ")"));
					}
					var res = bridgeInterface[method].apply(bridgeInterface, args);
					var event = win.document.createEvent("CustomEvent");
					event.initCustomEvent("firetvbridge-response", true, false, {
						"method" : method,
						"response" : res,
						"callback" : callback,
						"callbackArgs" : callbackArgs
					});
					win.dispatchEvent(event);
				} catch (e) {
					if (FBTrace.DBG_FIRETV_BRIDGE || FBTrace.DBG_FIRETV_ERROR) {
						FBTrace
								.sysout(bridge.CAT + "[bridgeHandler] ERROR - Error while calling method : " + method,
										e);
					}
				}
			} else {
				if (FBTrace.DBG_FIRETV_BRIDGE || FBTrace.DBG_FIRETV_ERROR) {
					FBTrace.sysout(bridge.CAT + "[bridgeHandler] ERROR - Unknown method : " + method);
				}
			}
		};

		var unloadHandler = function(event) {
			var win = event.target;
			win.removeEventListener("firetvbridge-call", bridgeHandler, false);
			win.removeEventListener("unload", unloadHandler, false);
			if (win.document) {
				win.document.documentElement.removeEventListener("DOMAttrModified", _this.TabManager.onAttrModified,
						false);
			}
			if (FBTrace.DBG_FIRETV_BRIDGE) {
				FBTrace.sysout(bridge.CAT + "[unregisterBridge()] " + win.location);
			}
		};

		var onDOMContentLoaded = function(event) {
			event.target.removeEventListener("DOMContentLoaded", onDOMContentLoaded, false);
			event.target.documentElement.addEventListener("DOMAttrModified", _this.TabManager.onAttrModified, false);
		};

		Delegate.prototype = {

			registerBridge : function(win) {
				win.addEventListener("unload", unloadHandler, false);
				win.addEventListener("firetvbridge-call", bridgeHandler, false);
				win.addEventListener("DOMContentLoaded", onDOMContentLoaded, false);
				if (FBTrace.DBG_FIRETV_BRIDGE) {
					FBTrace.sysout(bridge.CAT + "[registerBridge()] " + win.location);
				}
			},

			observe : function(aSubject, aTopic, aData) {

				if (aTopic == "content-document-global-created") {
					if (aSubject && aSubject.location
							&& (aSubject.location.href.match(/^http/) || aSubject.location.href.match(/^dvb/))) {
						if (_this.utils.configuredHost(aSubject.location)) {
							var win = aSubject;
							if (win == win.top) {
								this.registerBridge(win);
							} 
						}
					}
				}
			},
			QueryInterface : function(aIID) {
				if (aIID.equals(Ci.nsIObserver) || aIID.equals(Ci.nsISupports)) {
					return this;
				}
				throw Cr.NS_NOINTERFACE;
			}
		};

		bridge.init = function(loggingGroupId) {
			if (FBTrace.DBG_FIRETV_BRIDGE) {
				FBTrace.sysout(bridge.CAT + "[init]");
			}
			bridge.delegate = new Delegate();
			observerService.addObserver(bridge.delegate, "content-document-global-created", false);
			if (FBTrace.DBG_FIRETV_BRIDGE) {
				FBTrace.sysout(bridge.CAT + "[done]");
			}
		};

		bridge.shutdown = function() {
			if (FBTrace.DBG_FIRETV_BRIDGE) {
				FBTrace.sysout(bridge.CAT + "[shutdown]");
			}
			observerService.removeObserver(bridge.delegate, "content-document-global-created", false);
			delete bridge.delegate;
			if (FBTrace.DBG_FIRETV_BRIDGE) {
				FBTrace.sysout(bridge.CAT + "[done]");
			}
		};
		this.Bridge = bridge;

	}).apply(getFireTVPluginInstance());

} catch (exc) {
	alert("[ftv-bridge.js] " + exc);
}