/* See license.txt for terms of usage */

try {
	(function() {
		if (this.Broadcast) {
			return;
		}
		
		const Cc = Components.classes;
		const Ci = Components.interfaces;
		const Cr = Components.results;
		
		var _this = this;

		var FBTrace = this.FBTrace;
		
		var broadcast = {};

		broadcast.CAT = "[ftv.broadcast] ";

		broadcast.defaultChannelList = [];
		broadcast.customChannelList = [];

		broadcast.getChannelByCcid = function(ccid) {
			var channelList = (_this.PreferenceManager.customChannelListSelected())?broadcast.customChannelList:broadcast.defaultChannelList;
			var i;
			for (i = 0; i < channelList.length; i++) {
				if (channelList[i].ccid == ccid) {
					return channelList[i];
				}
			}
			return null;
		};

		broadcast.parseChannelList = function(url) {
			var chanListSrc = null;
			if (FBTrace.DBG_FIRETV_BROADCAST) {
				FBTrace.sysout(broadcast.CAT + "[parseChannelList] url : " + url);
			}
			try {
				chanListSrc = _this.FileUtils
					.getResourceAsString(url);
			} catch (e){
				if (FBTrace.DBG_FIRETV_BROADCAST || FBTrace.DBG_FIRETV_ERROR) {
					FBTrace.sysout(broadcast.CAT + "[parseChannelList] ERROR reading channel list file!", e);
				}
				throw _this.Messages.getMessage("firetv.channelList.unableToLoadFile");
			}
			var parser = new DOMParser();
			var chanList = [];
			var invalid = false;
			var details = "\n";
			var i;
			try {
				var chanListXml = parser.parseFromString(chanListSrc, "text/xml");
				if (FBTrace.DBG_FIRETV_BROADCAST) {
					FBTrace.sysout(broadcast.CAT + "[parseChannelList] channel list parsing result xml:", chanListXml);
				}
				// -- currently parseFromString does not generate error bu return a parsererror document
				// - see bug https://bugzilla.mozilla.org/show_bug.cgi?id=45566
				// Keeps the surrounding try/catch for future evolutions
				// Calls querySelector instead of looking documentElement tagName as it seems to be fetched asynchronously ??
				var xmldetails = chanListXml.querySelectorAll("parsererror, sourcetext");
				if (xmldetails.length > 0){
					invalid = true;
					for ( i = 0; i < xmldetails.length; i++) {
						details += xmldetails[i].textContent;
					}
				} else {
					var chan, channels = chanListXml.getElementsByTagName('Channel');
					for ( i = 0; i < channels.length; i++) {
						chan = channels[i];
						chanList.push({
							ccid : chan.getAttribute('ccid'),
							onid : chan.getElementsByTagName('ONID')[0].textContent,
							tsid : chan.getElementsByTagName('TSID')[0].textContent,
							sid : chan.getElementsByTagName('SID')[0].textContent,
							name : chan.getElementsByTagName('Name')[0].textContent
						});
					}
				}
			} catch(e){
				if (FBTrace.DBG_FIRETV_BROADCAST || FBTrace.DBG_FIRETV_ERROR) {
					FBTrace.sysout(broadcast.CAT + "[parseChannelList] ERROR while parsing channel list!", e);
					details = e;
				}
				invalid = true;
			}
			invalid = invalid || chanList.length == 0;
			if (invalid){
				throw _this.Messages.getMessage("firetv.channelList.invalidFormat") + details;
			}
			return chanList;
		};

		broadcast.init = function() {
			if (FBTrace.DBG_FIRETV_BROADCAST) {
				FBTrace.sysout(broadcast.CAT + "[init]");
			}
			broadcast.defaultChannelList = broadcast.parseChannelList("chrome://firetv-channels/content/ChannelList-fr.xml");
			if (_this.FileUtils.userChannelListFileExists()){
				broadcast.customChannelList = broadcast.parseChannelList("firetv://xml/channels");
			}
			if (FBTrace.DBG_FIRETV_BROADCAST) {
				FBTrace.sysout(broadcast.CAT + "[done]");
			}
		};

		broadcast.shutdown = function() {
			if (FBTrace.DBG_FIRETV_BROADCAST) {
				FBTrace.sysout(broadcast.CAT + "[shutdown]");
			}

			if (FBTrace.DBG_FIRETV_BROADCAST) {
				FBTrace.sysout(broadcast.CAT + "[done]");
			}
		};

		this.Broadcast = broadcast;

	}).apply(getFireTVPluginInstance());

} catch (exc) {
	alert("[ftv-broadcast.js] " + exc);
}