/* See license.txt for terms of usage */

try {
	(function() {
		if (this.PreferenceManager) {
			return;
		}

		const Cc = Components.classes;
		const Ci = Components.interfaces;
		const Cr = Components.results;
		
		var _this = this;
		
		var FBTrace = this.FBTrace;

		var prefService = Cc["@mozilla.org/preferences-service;1"].getService(
				Ci.nsIPrefService).getBranch("extensions.firetv.");

		var manager = {};

		manager.CAT = "[ftv.preferences] ";

		manager.getGlobalMarginDisplay = function() {
			return prefService.getBoolPref("marginDisplay");
		};
		
		manager.getDefaultMediaPlayer = function() {
			return prefService.getCharPref("defaultMediaPlayer");
		};

		manager.getGlobalDVBRoot = function() {
			var root = prefService.getCharPref("dvbRoot");
			try {
				var dvbRoot = Cc["@mozilla.org/file/local;1"]
						.createInstance(Ci.nsILocalFile);
				dvbRoot.initWithPath(root);
				if (dvbRoot.exists() && dvbRoot.isDirectory()) {
					return dvbRoot;
				}
			} catch (e) {
			}
			return null;
		};
		
		manager.setGlobalDVBRoot = function(dvbRoot) {
			prefService.setCharPref("dvbRoot", dvbRoot.path);
		};

		manager.customChannelListSelected = function() {
			return prefService.getBoolPref("customChannelListSelected") && _this.FileUtils.userChannelListFileExists();
		};
		
		manager.setCustomChannelListSelected = function(selected) {
			prefService.setBoolPref("customChannelListSelected", selected);
		};
		
		manager.getDefaultProfile = function() {
			return _this.ProfileManager.profilesNames[0];
		};

		manager.getHostRelatedPref = function(host, pref) {
			if (/^dvb:\//.test(host)) {
				host = "dvb:/";
			}
			var prefsString = prefService.getCharPref("hostsPrefs");
			var prefs = JSON.parse(prefsString);
			if (prefs[host] && prefs[host][pref]) {
				return prefs[host][pref];
			}
			return null;
		};

		manager.getAllHostRelatedPrefs = function(host) {
			if (/^dvb:\//.test(host)) {
				host = "dvb:/";
			}
			var res = [];
			var prefsString = prefService.getCharPref("hostsPrefs");
			var prefs = JSON.parse(prefsString);
			var pref;
			if (prefs[host]) {
				for (pref in prefs[host]) {
					if (prefs[host].hasOwnProperty(pref)) {
						res.push({
							"pref" : pref,
							"value" : prefs[host][pref]
						});
					}
				}
			}
			return res;
		};

		manager.setHostRelatedPref = function(host, pref, value) {
			if (/^dvb:\//.test(host)) {
				host = "dvb:/";
			}
			var prefsString = prefService.getCharPref("hostsPrefs");
			var prefs = JSON.parse(prefsString);
			if (!prefs[host]) {
				prefs[host] = {};
			}
			prefs[host][pref] = value;
			prefsString = JSON.stringify(prefs);
			prefService.setCharPref("hostsPrefs", prefsString);
		};

		manager.addConfiguredHost = function(host) {
			if (/^dvb:\//.test(host)) {
				host = "dvb:/";
			}
			var prefsString = prefService.getCharPref("hostsPrefs");
			var prefs = JSON.parse(prefsString);
			if (!prefs[host]) {
				prefs[host] = {};
				prefsString = JSON.stringify(prefs);
				prefService.setCharPref("hostsPrefs", prefsString);
				return true;
			}
			return false;
		};

		manager.removeConfiguredHost = function(host) {
			if (/^dvb:\//.test(host)) {
				return false;
			}
			var prefsString = prefService.getCharPref("hostsPrefs");
			var prefs = JSON.parse(prefsString);
			if (prefs[host]) {
				delete prefs[host];
				prefsString = JSON.stringify(prefs);
				prefService.setCharPref("hostsPrefs", prefsString);
				return true;
			}
			return false;
		};

		manager.getConfiguredHosts = function() {
			var hosts = [];
			var host;
			var prefsString = prefService.getCharPref("hostsPrefs");
			var prefs = JSON.parse(prefsString);
			var dvbVirtualHostDetected = false;
			for (host in prefs) {
				if (prefs.hasOwnProperty(host)) {
					if (host == "dvb:/") {
						dvbVirtualHostDetected = true;
					}
					hosts.push(host);
				}
			}
			if (!dvbVirtualHostDetected) {
				hosts.push("dvb:/");
			}
			return hosts;
		};

		manager.isConfiguredHost = function(host) {
			if (/^dvb:\//.test(host)) {
				host = "dvb:/";
			}
			var prefsString = prefService.getCharPref("hostsPrefs");
			var prefs = JSON.parse(prefsString);
			return (prefs[host]) ? true : false;
		};
		
		manager.init = function() {
			if (FBTrace.DBG_FIRETV_PREFERENCES) {
				FBTrace.sysout(manager.CAT + "[init]");
			}
			
			if (FBTrace.DBG_FIRETV_PREFERENCES) {
				FBTrace.sysout(manager.CAT + "[done]");
			}
		};

		manager.shutdown = function() {
			if (FBTrace.DBG_FIRETV_PREFERENCES) {
				FBTrace.sysout(manager.CAT + "[shutdown]");
			}
		
			delete requestObserver;
			if (FBTrace.DBG_FIRETV_PREFERENCES) {
				FBTrace.sysout(manager.CAT + "[done]");
			}
		};

		this.PreferenceManager = manager;

	}).apply(getFireTVPluginInstance());

} catch (exc) {
	alert("[ftv-preferences.js] " + exc);
}