/* See license.txt for terms of usage */

try {
	(function() {
		if (this.SVG) {
			return;
		}

		const Cc = Components.classes;
		const Ci = Components.interfaces;
		const Cr = Components.results;
		
		var _this = this;

		var FBTrace = this.FBTrace;
		
		var svg = {};

		svg.CAT = "[ftv.svg] ";

		svg.getTVImage = function(profile, host, options) {

			options = (options)?options:{};
			
			var displayTVImage = _this.FileUtils.userImageExists()
					&& (_this.PreferenceManager.getHostRelatedPref(host, "tvimage-display") == "true");
			var profileDefinition = _this.ProfileManager.getProfileDefinition(profile);
			if (FBTrace.DBG_FIRETV_SVG) {
				FBTrace.sysout(svg.CAT + "[getTVImage] for profile: " + profile + " [" + host + "] (displayTVImage="
						+ displayTVImage + ")", {profileDefinition:profileDefinition, options:options});
			}

			var tvFormat = _this.UI.getTVFormatById(profileDefinition.defaultTVFormat);
			var targetScaledWidth = 0, targetScaledHeight = 0;
//			if (tvFormat.label == "RAW") {
				targetScaledWidth = profileDefinition.resolution.width;
				targetScaledHeight = profileDefinition.resolution.height;
//			} else {
//				targetScaledWidth = profileDefinition.resolution.height * tvFormat.ratioX / tvFormat.ratioY;
//				targetScaledHeight = profileDefinition.resolution.width * tvFormat.ratioY / tvFormat.ratioX;
//			}

			var viewBox = "0 0 " + targetScaledWidth + " " + targetScaledHeight;
			if (FBTrace.DBG_FIRETV_SVG) {
				FBTrace.sysout(svg.CAT + "[getTVImage] viewBox: " + viewBox);
			}

			var scaleFactor = 0.5;
			var centerBigFireTVLogoTransform = "scale(" + scaleFactor + ") translate(" + (targetScaledWidth / 2) + ","
					+ (targetScaledHeight / 2) + ")";

			scaleFactor = 0.1;
			var marginFactor = 0.05;
			var translateX = Math.round(targetScaledWidth - (scaleFactor * targetScaledWidth)
					- (marginFactor * targetScaledWidth));
			var translateY = Math.round(marginFactor * targetScaledHeight);
			var topRightSmallFireTVLogoTransform = "translate(" + translateX + "," + translateY + ") scale("
					+ scaleFactor + ")";

			if (FBTrace.DBG_FIRETV_SVG) {
				FBTrace.sysout(svg.CAT + "[getTVImage] centerBigFireTVLogoTransform: " + centerBigFireTVLogoTransform);
				FBTrace.sysout(svg.CAT + "[getTVImage] topRightSmallFireTVLogoTransform: "
						+ topRightSmallFireTVLogoTransform);
			}

			var ANIM_DELAY = 12000;

			var s = '<?xml version="1.0" encoding="utf-8"?>\n';
			s += '<svg xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" \n';
			s += '         width="100%" height="100%" \n';
			if(options.preserveAspectRatio == "false"){
				s += '         preserveAspectRatio="none"\n';
			} else {
				// -- preserve aspect ratio by default
				s += '         preserveAspectRatio="xMidYMid meet"\n';
			}
			s += '         viewBox="' + viewBox + '" \n';
			s += '         >\n';

			s += '   <script type="text/javascript">\n';
			s += '   <![CDATA[ \n';
			s += '      var timeoutId = -1;\n';
			s += '      var delay = ' + ANIM_DELAY + ';\n';
			s += '      var anim = function(){\n';
			s += '         var anim1 = document.getElementById("anim1");\n';
			s += '         var anim2 = document.getElementById("anim2");\n';
			s += '         anim1.beginElement();\n';
			s += '         anim2.beginElement();\n';
			s += '         timeoutId = setTimeout(anim, delay);\n';
			s += '      };\n';

			s += '      var toggle = function(){\n';
			s += '         if (timeoutId == -1){\n';
			s += '            timeoutId = setTimeout(anim, delay);\n';
			s += '         } else {\n';
			s += '            clearTimeout(timeoutId);\n';
			s += '            timeoutId = -1;\n';
			s += '         }\n';
			s += '      };\n';
			
			s += '      var loadHandler = function(event){\n';
			s += '        toggle();\n';
			if(!options.hasOwnProperty("noChannel")){
				s += '       setTopLeftLabel("' + options.channelDisplayName.replace(/"/g, '\\"') + '");\n';	
			}
			s += '      };\n';
			
			s += '      var SVG_NS = "http://www.w3.org/2000/svg";\n';
			
			s += '      var setTopLeftLabel = function(label) {\n';
			s += '     		var padding = 10;\n';
			s += '      	var fontSize = 50;\n';
			s += '      	var topLeftLabelDisplay = document.getElementById("topLeftLabelDisplay");\n';
			s += '      	if (topLeftLabelDisplay){\n';
			s += '      		topLeftLabelDisplay.parentNode.removeChild(topLeftLabelDisplay);\n';
			s += '      	}\n';
			    				
			s += '      	if (!label || label == "") {\n';
			s += '      		return;\n';
			s += '      	}\n';
				
			s += '      	topLeftLabelDisplay = document.createElementNS(SVG_NS, "g");\n';
			s += '      	topLeftLabelDisplay.setAttributeNS(null, "id", "topLeftLabelDisplay");\n';
			s += '      	topLeftLabelDisplay.setAttributeNS(null, "transform", "translate(30,30)");\n';

				
			s += '      	var text = document.createElementNS(SVG_NS, "text");\n';
			s += '      	text.setAttributeNS(null, "x", padding);\n';
			s += '      	text.setAttributeNS(null, "y", fontSize*1.7/2);\n';
			s += '      	text.setAttributeNS(null, "alignment-baseline", "baseline");\n';
			s += '      	text.setAttributeNS(null, "dominant-baseline", "central");\n';
			s += '      	text.setAttributeNS(null, "font-size", fontSize + "px");\n';
			s += '      	text.setAttributeNS(null, "font-family", "FireTV");\n';
			s += '      	text.setAttributeNS(null, "fill", "#FFFFFF");\n';
			s += '      	text.setAttributeNS(null, "fill-opacity", "0.5");\n';
			s += '      	text.setAttributeNS(null, "text-anchor", "left");\n';
			s += '      	text.appendChild(document.createTextNode(label));\n';
			s += '      	topLeftLabelDisplay.appendChild(text);\n';
				
				
			s += '      	document.documentElement.appendChild(topLeftLabelDisplay);\n';
				
			s += '      	// -- update background;\n';
			s += '      	var svgRect = text.getBBox();\n';
			s += '      	var rect = document.createElementNS(SVG_NS, "rect");\n';
			s += '      	rect.setAttributeNS(null, "x", 0);\n';
			s += '      	rect.setAttributeNS(null, "y", 0);\n';
			s += '      	rect.setAttributeNS(null, "width", (svgRect.width + 2*padding));\n';
			s += '      	rect.setAttributeNS(null, "height", (svgRect.height + 2*padding));\n';
			s += '      	rect.setAttributeNS(null, "stroke", "none");\n';
			s += '      	rect.setAttributeNS(null, "stroke-width", "0");\n';
			s += '      	rect.setAttributeNS(null, "fill", "#444444");\n';
			s += '      	rect.setAttributeNS(null, "fill-opacity", "0.5");\n';
			s += '      	rect.setAttributeNS(null, "rx", "10");\n';
			s += '      	rect.setAttributeNS(null, "ry", "10");\n';
			
			s += '      	topLeftLabelDisplay.insertBefore(rect, text);\n';
			s += '      };\n';
			
			s += '      window.addEventListener("load", loadHandler, false);\n';
			s += '      window.addEventListener("click", toggle, false);\n';

			s += _this.FileUtils.getResourceAsString("chrome://firetv-profile/content/common/mediaplayer/svg-api.js");

			s += '   ]]>\n';
			s += '	 </script>\n';

			s += '	 <defs>\n';

			s += '	    <filter id="InnerShadow" >\n';
			s += '	    	 <feGaussianBlur stdDeviation="6" />\n';
			s += '	    	 <feComposite\n';
			s += '	    	 	 operator="arithmetic"\n';
			s += '	    	 	 in2="SourceAlpha"\n';
			s += '	    	 	 k2="-1"\n';
			s += '	    	 	 k3="1" />\n';
			s += '	    </filter> \n';

			s += '	    <g id="FireTV">\n';
			s += '         <svg width="100%" height="100%" viewBox="0 0 200 130" preserveAspectRatio="xMidYMid meet">\n';
			s += '	    	  <rect x="0" y="0" width="200" height="130" stroke="none" stroke-width="0"\n';
			s += '	    	 	 fill="#FFFFFF" fill-opacity="0.3" rx="15" ry="15"/>\n';
			s += '	    	  <rect x="20" y="20" width="160" height="80" stroke="none" stroke-width="0"\n';
			s += '	    	 	 fill="#444444" fill-opacity="0.5" rx="15" ry="15"/>\n';
			s += '	    	  <rect x="50" y="105" width="100" height="10" stroke="none" stroke-width="0"\n';
			s += '	    	 	 fill="#444444" fill-opacity="0.5" rx="3" ry="3"/>\n';
			s += '	    	  <text y="60" x="100"  alignment-baseline="baseline" dominant-baseline="central"  font-size="55px"  font-family="FireTV" fill="#FFFFFF" fill-opacity="0.5" text-anchor="middle" >FireTV</text>\n';
			s += '         </svg>\n';
			s += '	    </g>\n';

			s += '	    <g id="TVFrame">\n';
			s += '	    	 <rect x="0" y="0" width="100%" height="100%" stroke="#343739" stroke-width="0" stroke-opacity="0.5"\n';
			s += '	    	 	 fill="#FFFFFF" fill-opacity="0.8" filter="url(#InnerShadow)"/>\n';
			s += '	    	 <rect x="0" y="0" width="100%" height="100%" stroke="none" stroke-width="0"\n';
			s += '	    	 	 fill="#000000" fill-opacity="0.2"/>\n';
			s += '	    </g>\n';

			s += '	    <pattern id="BackgroundPattern"\n';
			s += '	    	 x="0" y="0" width="40" height="40" patternUnits="userSpaceOnUse"\n';
			s += '	    	 >\n';
			s += '	    	 <rect x="0" y="0" width="40" height="40"\n';
			s += '	    		  fill="#3B3E40" stroke="none"/>\n';
			s += '	    	 <path d="M 0 20 L 0 40 L 40 0 L 20 0 z"\n';
			s += '		    	  fill="#343739" stroke="none" stroke-width="0" />\n';
			s += '	    	 <path d="M 20 40 L 40 20 L 40 40 z"\n';
			s += '	    	 	 fill="#343739" stroke="none" stroke-width="0" />\n';
			s += '	    </pattern>\n';

			s += '	    <pattern id="BackgroundReversePattern"\n';
			s += '	    	 x="0" y="0" width="40" height="40" patternUnits="userSpaceOnUse"\n';
			s += '	    	 >\n';
			s += '	    	 <rect x="0" y="0" width="40" height="40"\n';
			s += '	    	 	 fill="#3B3E40" stroke="none"/>\n';
			s += '	    	 <path d="M 0 0 L 20 0 L 40 20 L 40 40 z"\n';
			s += '	    	 	 fill="#343739" stroke="none" stroke-width="0" />\n';
			s += '	    	 <path d="M 0 20 L 20 40 L 0 40 z"\n';
			s += '	    	 	 fill="#343739" stroke="none" stroke-width="0" />\n';
			s += '	    </pattern>\n';
			
			// -- player controls
			s += '		<g id="Stop">\n';
			s += '	   		<svg width="100" height="50" viewBox="0 0 100 50" preserveAspectRatio="xMidYMid meet">\n';
			s += '	    		<rect x="0" y="0" width="100%" height="100%" stroke="none"\n';
			s += '	    			stroke-width="0" fill="#444444" fill-opacity="0.5" rx="15" ry="15"  />\n';
			s += '	    		<path d="M 32 7 L 68 7 L 68 43 L 32 43 z" fill="#FFFFFF" fill-opacity="0.5" stroke="none"\n';
			s += '	    			stroke-width="0" />\n';
			s += '	    	</svg>\n';
			s += '	    </g>\n';
			s += '	    <g id="Play">\n';
			s += '	    	<svg width="100" height="50" viewBox="0 0 100 50" preserveAspectRatio="xMidYMid meet">\n';
			s += '	    		<rect x="0" y="0" width="100%" height="100%" stroke="none"\n';
			s += '	    			stroke-width="0" fill="#444444" fill-opacity="0.5" rx="15" ry="15"  />\n';
			s += '	    		<path d="M 36 7 L 67 25 L 36 43 z" fill="#FFFFFF" fill-opacity="0.5" stroke="none"\n';
			s += '	    			stroke-width="0" />\n';
			s += '	    	</svg>\n';
			s += '	    </g>\n';
			s += '	    <g id="Pause">\n';
			s += '			<svg width="100" height="50" viewBox="0 0 100 50" preserveAspectRatio="xMidYMid meet">\n';
			s += '	   			<rect x="0" y="0" width="100%" height="100%" stroke="none"\n';
			s += '	   				stroke-width="0" fill="#444444" fill-opacity="0.5" rx="15" ry="15"  />\n';
			s += '	    		<path d="M 32 7 L 44 7 L 44 43 L 32 43 z" fill="#FFFFFF" fill-opacity="0.5" stroke="none"\n';
			s += '	   				stroke-width="0" />\n';
			s += '	    		<path d="M 56 7 L 68 7 L 68 43 L 56 43 z" fill="#FFFFFF" fill-opacity="0.5" stroke="none"\n';
			s += '	    			stroke-width="0" />\n';
			s += '	    	</svg>\n';
			s += '	    </g>\n';
			s += '	    <g id="FFwd">\n';
			s += '	    	<svg width="100" height="50" viewBox="0 0 100 50" preserveAspectRatio="xMidYMid meet">\n';
			s += '	    		<rect x="0" y="0" width="100%" height="100%" stroke="none"\n';
			s += '	    			stroke-width="0" fill="#444444" fill-opacity="0.5" rx="15" ry="15"  />\n';
			s += '	    		<path d="M 23 7 L 54 25 L 23 43 z" fill="#FFFFFF" fill-opacity="0.5" stroke="none"\n';
			s += '	    			stroke-width="0" />\n';
			s += '	    		<path d="M 53 7 L 84 25 L 53 43 z" fill="#FFFFFF" fill-opacity="0.5" stroke="none"\n';
			s += '	    			stroke-width="0" />\n';
			s += '	    	</svg>\n';
			s += '	    </g>\n';
			s += '	    <g id="FRwd">\n';
			s += '	    	<svg width="100" height="50" viewBox="0 0 100 50" preserveAspectRatio="xMidYMid meet">\n';
			s += '	    		<g transform="scale(-1) translate(-100,-50)">\n';
			s += '	    			<use xlink:href="#FFwd"></use>\n';
			s += '	    		</g>\n';
			s += '	    	</svg>\n';
			s += '	    </g>\n';
					
			s += '	    <g id="PlayerControlDef">\n';
			s += '	    	<svg width="100%" height="100" preserveAspectRatio="none">\n';
			s += '	    		<rect x="0" y="0" width="100%" height="100%" stroke="none"\n';
			s += '	    			stroke-width="0" fill="#FFFFFF" fill-opacity="0.3" />\n';
			s += '	    		<g transform="translate(35,10)">\n';
			s += '	    			<use id="videoStatus" />\n';
			s += '	    		</g>\n';
			s += '	    		<text id="videoPlayPosition" y="80" x="10" alignment-baseline="baseline"\n';
			s += '	    			dominant-baseline="central" font-size="25px" font-family="FireTV"\n';
			s += '	    			fill="#FFFFFF" fill-opacity="0.5" text-anchor="left"></text>\n';
			s += '	    		<foreignObject y="10" x="170" width="100%" height="100">\n';
			s += '	    			<body xmlns="http://www.w3.org/1999/xhtml" style="background-color:transparent; width:100%; height:100%; padding:0; margin: 0">\n';
			s += '	    				<div id="videoUrl" style="background-color:transparent; color:rgba(255,255,255,0.5); word-wrap: break-word;  font-size: 25px; font-family: FireTV; position: absolute; top:0; left:0; height:50px; right:170px; ">\n';
			s += '	    				</div>\n';
			s += '	    			</body>\n';
			s += '	    		</foreignObject>\n';
			s += '	    	</svg>\n';
			s += '	    </g>\n';
					
			s += '	 </defs>\n';

			s += '	 <rect x="0" y="0" width="100%" height="100%" fill="url(#BackgroundReversePattern)" stroke="none" stroke-width="0"/>\n';
			if (displayTVImage) {
				s += '	 <image width="100%" height="100%" preserveAspectRatio="xMidYMid meet" xlink:href="firetv://img/tv"/>\n';
			}
			s += '	 <g>\n';
			s += '	 	<use  xlink:href="#TVFrame" />\n';
			s += '	 </g>\n';
			if (displayTVImage) {
				s += '	 <g transform="' + topRightSmallFireTVLogoTransform + '">\n';
			} else {
				s += '	 <g transform="' + centerBigFireTVLogoTransform + '">\n';
			}
			s += '	 	<use  xlink:href="#FireTV" >\n';
			s += '	 	   <animateTransform id="anim1" begin="indefinite" restart="whenNotActive" attributeName="transform" attributeType="XML"\n';
			s += '	 	      type="translate" values="0,0; ' + (targetScaledWidth / 2) + ',0; ' + (targetScaledWidth)
					+ ',0; ' + (targetScaledWidth / 2) + ',0; 0,0" dur="0.8s"\n';
			s += '	 	      additive="sum" fill="freeze"/>\n';
			s += '	 	   <animateTransform id="anim2" begin="indefinite" restart="whenNotActive" attributeName="transform" attributeType="XML"\n';
			s += '	 	      type="scale"     values="1,1; 0.0,1; -1,1; 0.0,1; 1,1" dur="0.8s"\n';
			s += '	 	      additive="sum" fill="freeze"/>\n';
			s += '      </use>\n';
			s += '	 </g>\n';

			s += '</svg>\n';
			if (FBTrace.DBG_FIRETV_SVG) {
				FBTrace.sysout(svg.CAT + "[getTVImage] generated (size=" + s.length + "): ", s);
			}
			return s;
		};

		svg.init = function() {
			if (FBTrace.DBG_FIRETV_SVG) {
				FBTrace.sysout(svg.CAT + "[init]");
			}

			if (FBTrace.DBG_FIRETV_SVG) {
				FBTrace.sysout(svg.CAT + "[done]");
			}

		};

		svg.shutdown = function() {
			if (FBTrace.DBG_FIRETV_SVG) {
				FBTrace.sysout(svg.CAT + "[shutdown]");
			}

			if (FBTrace.DBG_FIRETV_SVG) {
				FBTrace.sysout(svg.CAT + "[done]");
			}

		};

		this.SVG = svg;

	}).apply(getFireTVPluginInstance());
} catch (exc) {
	alert("[ftv-svg.js] " + exc);
}
