/* See license.txt for terms of usage */

/*global FireTVPlugin, Node, window, document */
(function() {

	var nodeToString = function(node) {
		var res = "";
		res += node.nodeName.toLowerCase();
		if (node.id) {
			res += "#" + node.id;
		}
		if (node.className) {
			var classes = node.className.split(/\s+/);
			var j;
			for (j = 0; j < classes.length; j++) {
				res += "." + classes[j];
			}
		}
		return res;
	};

	var getCustomCSSValue = function(node, property) {
		var uri = window.location.href;
		var msg = "[getCustomCSSValue] for " + property + " on " + nodeToString(node) + " (" + uri + ")";
		
		var customSheet = FireTVPlugin.customCSSSheet;
		if (customSheet) {
			value = customSheet.getComputedValue(node, property);
			if (value !== null) {
				msg += " return : " + value;
				// console.info(msg);
				return value;
			}
		}
		var value = node.getAttribute("firetv-" + property);
		if (value && value !== "") {
			msg += " return : " + value;
			// console.info(msg)
			return value;
		}
		
		msg += " return : null";
		// console.info(msg)
		return null;
	};

	var context = {
		"nodeToString" : nodeToString,
		"getCustomCSSValue" : getCustomCSSValue
	};

	var overloadCSSStyleDeclaration = function(event) {
		if(event){
			event.target.removeEventListener("DOMContentLoaded", overloadCSSStyleDeclaration, false);
		}
		var HTML_ELEMENTS = FireTVPlugin.constants.HTML_ELEMENTS;
		var elClass, property, i;
		for (elClass in HTML_ELEMENTS) {
			if (HTML_ELEMENTS.hasOwnProperty(elClass)) {
				if (window[HTML_ELEMENTS[elClass]] && window[HTML_ELEMENTS[elClass]].prototype) {
					var g = window[HTML_ELEMENTS[elClass]].prototype.__lookupGetter__("style");
					if (g) {
						window[HTML_ELEMENTS[elClass]].prototype.__defineGetter__("style",
								function() {
									var res = g.apply(this, []);
									FireTVPlugin.lastStyleAccessOnNode = this;
									return res;
								});
						window[HTML_ELEMENTS[elClass]].prototype.__defineGetter__("nativestyle", function() {
							return g.apply(this, []);
						});
					}
				}
			}
		}

		var nativeGetComputedStyle = window.getComputedStyle;
		window.getComputedStyle = function(elt, pseudoElt) {
			var s = nativeGetComputedStyle.apply(window, [ elt, pseudoElt ]);
			if (pseudoElt === "from-binding") {
				return s;
			}
			FireTVPlugin.lastStyleAccessOnNode = elt;
			var val;
			var o = {};
			var count = 0;
			var i;
			var property;
			for (property in FireTVPlugin.customCSSHooks) {
				if (FireTVPlugin.customCSSHooks.hasOwnProperty(property)) {
					val = getCustomCSSValue(elt, property);
					if (val !== null) {
						delete s[property];
						o[property] = "" + val;
						count++;
					}
				}
			}
			if (count > 0) {
				// return a proxy object because s cannot be modified
				// - filter with !o.hasOwnProperty to get everything that is not
				// in o
				for (i in s) {
					if (!o.hasOwnProperty(i)) {
						o[i] = s[i];
					}
				}
				o.original = s;
				return o;
			}
			return s;
		};

		for (property in FireTVPlugin.customCSSHooks) {
			if (FireTVPlugin.customCSSHooks.hasOwnProperty(property)) {
				(function() {
					var getter = FireTVPlugin.customCSSHooks[property].getter;
					var setter = FireTVPlugin.customCSSHooks[property].setter;
					if (getter) {
						window.CSSStyleDeclaration.prototype.__defineGetter__(property, (function() {
							return function() {
								return getter.apply(context, []);
							}
						})());
					}
					if (setter) {
						window.CSSStyleDeclaration.prototype.__defineSetter__(property, (function() {
							return function(val) {
								setter.apply(context, [ val ]);
							}
						})());
					}
				})();
			}
		}

		
		var replacements = FireTVPlugin.profileDefinition.cssReplacements;
		for (i=0; i<replacements.length; i++) {
			(function() {
				var prop = replacements[i].property;
				var val = replacements[i].value;
				var replacement = replacements[i].replacement;
				var origSetter = window.CSSStyleDeclaration.prototype.__lookupSetter__(prop);
				var setter = function(value){
					if (value == val){
						origSetter.apply(this, [replacement]);
						return replacement;
					} else {
						return origSetter.apply(this, [value]);
					}
				};
				window.CSSStyleDeclaration.prototype.__defineSetter__(prop, (function() {
					return function(val) {
						setter.apply(this, [ val ]);
					}
				})());
			})();
		}
		
		var nativeSetProperty = window.CSSStyleDeclaration.prototype.setProperty;
		window.CSSStyleDeclaration.prototype.setProperty = function(propertyName, value, priority) {
			if (propertyName in FireTVPlugin.customCSSHooks && FireTVPlugin.customCSSHooks[propertyName].setter) {
//				console.log(FireTVPlugin.lastStyleAccessOnNode);
//				console.log(propertyName + " : " + value);
				FireTVPlugin.customCSSHooks[propertyName].setter.apply(context, [ value ]);
			} else {
				nativeSetProperty.apply(this, [ propertyName, value, priority ]);
			}
		};
		
		var nativeGetPropertyPriority = window.CSSStyleDeclaration.prototype.getPropertyPriority;
		window.CSSStyleDeclaration.prototype.getPropertyPriority = function(propertyName) {
			return nativeGetPropertyPriority.apply(this, [ propertyName ]);
		};
		
		var nativeGetPropertyValue = window.CSSStyleDeclaration.prototype.getPropertyValue;
		window.CSSStyleDeclaration.prototype.getPropertyValue = function(propertyName) {
			var res;
			try {
				res = nativeGetPropertyValue.apply(this, [ propertyName ]);
			} catch (e){
				res = nativeGetPropertyValue.apply(this.original, [ propertyName ]);
			}
			return res;
		};
		
		var nativeRemoveProperty = window.CSSStyleDeclaration.prototype.removeProperty;
		window.CSSStyleDeclaration.prototype.removeProperty = function(propertyName) {
			return nativeRemoveProperty.apply(this, [ propertyName ]);
		};
	};

	var domAttrModifiedHandler = function (event) {
		if (event.target !== event.currentTarget) {
			return;
		}
		var element = event.target;
		var styleModified = event.attrName === "style";
		var classModified = event.attrName === "class";
		// var customModified = event.attrName.substr(0, "firetv-".length) ===
		// "firetv-";
		var i, property;
		if (styleModified) {
			var s = event.newValue.substring(0, event.newValue.length - 1);
			var properties = s.split(";");
			for (i = 0; i < properties.length; i++) {
				var kv = properties[i].split(":");
				if (kv.length === 2) {
					kv[0] = kv[0].replace(/^\s+/, "").replace(/\s+$/, "");
					kv[1] = kv[1].replace(/^\s+/, "").replace(/\s+$/, "");
					// console.info("style changed : " + kv[0] + " : " + kv[1]);
					for (property in FireTVPlugin.customCSSHooks) {
						if (FireTVPlugin.customCSSHooks.hasOwnProperty(property)) {
							if (FireTVPlugin.customCSSHooks[property].listenTo.indexOf(kv[0]) > -1) {
								FireTVPlugin.customCSSHooks[property].callback.apply(context, [
										element,
										property,
										property ]);
							}

						}
					}
				}
			}
		} else if (classModified) {
			for (property in FireTVPlugin.customCSSHooks) {
				if (FireTVPlugin.customCSSHooks.hasOwnProperty(property)) {
					FireTVPlugin.customCSSHooks[property].callback.apply(context, [
							element,
							property,
							"[class " + event.prevValue + "->" + event.newValue + "]" ]);
				}
			}
		}
		// else if (customModified) {
		// // var property =
		// // event.attrName.substr(css.CUSTOM_PROPERTY_PREFIX.length);
		//
		// // _this.Console.debug([css.CAT, "custom changed ("+property+"):
		// // " + event.prevValue + " -> " + event.newValue]);
		// }
	};

	var domNodeInsertedHandler = function(event) {
		// -- ignore <object> and <embed> as it seems to crash quicktime 
		if(event.originalTarget.tagName == "object" || event.originalTarget.tagName == "embed" ){
			return;
		}
		// -- the event seems to be fired only on the topmost element, not
		// its children
		var element = event.target;
		var elements = [];
		var i, property;
		if (element.nodeType === Node.ELEMENT_NODE) {
			elements.push(element);
			var children = element.getElementsByTagName("*");
			for (i = 0; i < children.length; i++) {
				elements.push(children[i]);
			}
		}
		for (i = 0; i < elements.length; i++) {
			if (elements[i].nodeName.toLowerCase() === "iframe" || elements[i].nodeName.toLowerCase() === "html"
					|| elements[i].nodeName.toLowerCase() === "head" || elements[i].nodeName.toLowerCase() === "meta"
					|| elements[i].nodeName.toLowerCase() === "script" || elements[i].nodeName.toLowerCase() === "object"
					|| elements[i].nodeName.toLowerCase() === "embed"|| elements[i].nodeName.toLowerCase() === "style") {
				continue;
			}
			element = elements[i];
			// console.info("[domNodeInsertedHandler()] " +
			// nodeToString(element))
			for (property in FireTVPlugin.customCSSHooks) {
				if (FireTVPlugin.customCSSHooks.hasOwnProperty(property)) {
					FireTVPlugin.customCSSHooks[property].callback.apply(context, [ element, property, "[init]" ]);
				}
			}
			element.addEventListener("DOMAttrModified", domAttrModifiedHandler, false);
		}
	};

	var domNodeRemovedHandler = function(event) {
		var element = event.target;
		if (element.nodeName.toLowerCase() === "iframe" || element.nodeName.toLowerCase() === "html"
			|| element.nodeName.toLowerCase() === "head" || element.nodeName.toLowerCase() === "meta"
			|| element.nodeName.toLowerCase() === "script" || element.nodeName.toLowerCase() === "object"
			|| element.nodeName.toLowerCase() === "embed"|| element.nodeName.toLowerCase() === "style") {
			return;
		}
		// console.info("[domNodeRemovedHandler()] " + nodeToString(element));
		element.removeEventListener("DOMAttrModified", domAttrModifiedHandler, false);
	};

	var shutdownSpecificRulesHandler = function(event) {
		event.target.removeEventListener("unload", shutdownSpecificRulesHandler, true);
		// console.info("Removing previous specific rules handlers...(" +
		// window.location.href + ")");
		document.removeEventListener("DOMNodeInserted", domNodeInsertedHandler, false);
		document.removeEventListener("DOMNodeRemoved", domNodeRemovedHandler, false);
	};

	var initSpecificRulesHandler = function(event) {
		if(event){
			event.target.removeEventListener("DOMContentLoaded", initSpecificRulesHandler, false);
		}
		// console.info("Initializing specific rules handlers... (" +
		// window.location.href + ")");
		window.addEventListener("unload", shutdownSpecificRulesHandler, true);

		var elements = document.querySelectorAll("*");
		var i, property, l = elements.length;
		for (i = 0; i < l; i++) {
			if (elements[i].nodeName.toLowerCase() === "iframe" || elements[i].nodeName.toLowerCase() === "html"
					|| elements[i].nodeName.toLowerCase() === "head" || elements[i].nodeName.toLowerCase() === "meta"
					|| elements[i].nodeName.toLowerCase() === "script" || elements[i].nodeName.toLowerCase() === "object"
					|| elements[i].nodeName.toLowerCase() === "embed"|| elements[i].nodeName.toLowerCase() === "style") {
				continue;
			}
			for (property in FireTVPlugin.customCSSHooks) {
				if (FireTVPlugin.customCSSHooks.hasOwnProperty(property)) {
					FireTVPlugin.customCSSHooks[property].callback.apply(context, [ elements[i], property, "[init]" ]);
				}
			}
			elements[i].addEventListener("DOMAttrModified", domAttrModifiedHandler, false);
		}
		
	};

	overloadCSSStyleDeclaration();
	window.addEventListener("DOMContentLoaded", initSpecificRulesHandler, false);
	document.addEventListener("DOMNodeInserted", domNodeInsertedHandler, false);
	document.addEventListener("DOMNodeRemoved", domNodeRemovedHandler, false);
})();
