/* See license.txt for terms of usage */

(function() {

	var debug = function(msg) {
		var i;
		var args = [ "[FireTVMediaPlayer." + debug.caller.name + "(" ];
		for (i = 0; i < debug.caller.arguments.length; i++) {
			args[0] += debug.caller.arguments[i];
			args.push(debug.caller.arguments[i]);
			if (i < debug.caller.arguments.length - 1) {
				args[0] += ", ";
			}
		}
		args[0] += ")]";
		for (i = 0; i < arguments.length; i++) {
			args.push(arguments[i]);
		}
		console.log.apply(console, args);
	};

	function FireTVMediaPlayer() {
	}

	var mediaPlayerPrototype = {
		"_impl" : {
			get : function() {
				var impl = FireTVPlugin.getPref("media-player");
				return FireTVPlugin[impl];
			}
		},
		"setData" : {
			value : function setData(target, val) {
				this._impl.setData(target, val);
			},
			enumerable : true
		},

		// -- real media player API
		"getPlayPosition" : {
			value : function getPlayPosition(target) {
				return this._impl.getPlayPosition(target);
			},
			writable : false,
			enumerable : true
		},
		"getPlayTime" : {
			value : function getPlayTime(target) {
				return this._impl.getPlayTime(target);
			},
			writable : false,
			enumerable : true
		},
		"getPlayState" : {
			value : function getPlayState(target) {
				return this._impl.getPlayState(target);
			},
			writable : false,
			enumerable : true
		},
		"getError" : {
			value : function getError(target) {
				return this._impl.getError(target);
			},
			enumerable : true
		},
		"getSpeed" : {
			value : function getSpeed(target) {
				return this._impl.getSpeed(target);
			},
			writable : false,
			enumerable : true
		},
		"getOnPlayStateChange" : {
			value : function getOnPlayStateChange(target) {
				return this._impl.getOnPlayStateChange(target);
			},
			writable : false,
			enumerable : true
		},
		"setOnPlayStateChange" : {
			value : function setOnPlayStateChange(target, val) {
				return this._impl.setOnPlayStateChange(target, val);
			},
			writable : false,
			enumerable : true
		},
		"play" : {
			value : function play(target, speed) {
				if (typeof speed === "undefined"){
					speed = 1;
				}
				this._impl.play(target, speed);
			},
			writable : false,
			enumerable : true
		},
		"stop" : {
			value : function stop(target) {
				this._impl.stop(target);
			},
			writable : false,
			enumerable : true
		},
		"seek" : {
			value : function seek(target, positionInMilliseconds) {
				return this._impl.seek(target, positionInMilliseconds);
			},
			writable : false,
			enumerable : true
		},
		"queue" : {
			value : function queue(target, uri) {
				return this._impl.queue(target, uri);
			},
			writable : false,
			enumerable : true
		},
		"getFullScreen" : {
			value : function getFullScreen(target) {
				return this._impl.getFullScreen(target);
			},
			writable : false,
			enumerable : true
		},
		"getOnFullScreenChange" : {
			value : function getOnFullScreenChange(target) {
				return this._impl.getOnFullScreenChange(target);
			},
			writable : false,
			enumerable : true
		},
		"setOnFullScreenChange" : {
			value : function setOnFullScreenChange(target, val) {
				return this._impl.setOnFullScreenChange(target, val);
			},
			writable : false,
			enumerable : true
		},
		"setFullScreen" : {
			value : function setFullScreen(target, fullScreen) {
				this._impl.setFullScreen(target, fullScreen);
			},
			writable : false,
			enumerable : true
		}
	};

	Object.defineProperties(FireTVMediaPlayer.prototype, mediaPlayerPrototype);

	FireTVPlugin.MediaPlayer = new FireTVMediaPlayer();

	Object.defineProperty(FireTVPlugin, "getBounds",
			{
				value : function getBounds(elt) {
					var res = {};
					// -- get current top/left
					var vtop = 0, vleft = 0, element = elt;
					do {
						vtop += element.offsetTop || 0;
						vleft += element.offsetLeft || 0;
						element = element.offsetParent;
					} while (element);

					// -- get current scale
					var transform = window.getComputedStyle(document.body, null)["MozTransform"].replace(/^matrix\(/,
							"").replace(/\)$/, "").split(",");
					var scaleX = parseFloat(transform[0]);
					var scaleY = parseFloat(transform[3]);

					// - get current deltaX (body left)
					var bodyLeft = window.getComputedStyle(document.body, null)["left"].replace(/px$/, "") >> 0;

					// - get current deltaY (documentElement paddingTop
					// -> space reserve for
					// plugin top bar)
					var paddingTop = window.getComputedStyle(document.documentElement, null).paddingTop.replace(/px$/,
							"") >> 0;

					// -- build res
					res.top = vtop;
					res.left = vleft;
					res.width = elt.offsetWidth;
					res.height = elt.offsetHeight;
					res.scaleX = scaleX;
					res.scaleY = scaleY;
					res.offsetTop = paddingTop;
					res.offsetLeft = bodyLeft;
					return res;
				},
				writable : false,
				enumerable : false
			});
})();