/* See license.txt for terms of usage */

/**
 * This is quiet complex but it should work as long as we do not use more than 
 * one instance of <object> of a given mime-type - which is the case in most commons situation.
 * It should work with multiple instance but does not because of : https://bugzilla.mozilla.org/show_bug.cgi?id=690259
 */
(function() {

	var debug = function(msg) {
		var message = (msg) ? msg : "";
		var i;
		var args = "(";
		for (i = 0; i < debug.caller.arguments.length; i++) {
			args += debug.caller.arguments[i];
			if (i < debug.caller.arguments.length - 1) {
				args += ", ";
			}
		}
		args += ")";
		console.log("[ObjectHelper." + debug.caller.name + args + "] " + message);
	};

	var typeToId = function(type) {
		if (typeof type == "string") {
			return type.replace("/", "_").replace("+", "_");
		}
		return "";
	};

	var proxifiedMimeTypes = [];
	var proxyMimeSuffix = "+firetv";
	var proxyMimeSuffixRegexp = new RegExp("\\+firetv$");

	function createProxy(object) {
		// -- replace original node by a new one modified to avoid original
		// mimetype plugin loading if any
		var proxyId = "firetvProxy" + new String(Math.random()).substring(2);
		
		var isEmbed = object instanceof HTMLEmbedElement;
		var dataAttribute = (isEmbed)?"src":"data";
		
		var data = object.getAttribute(dataAttribute);
		while(object.firstChild){
			object.removeChild(object.firstChild);
		}
		object.removeAttribute(dataAttribute);
		
		var proxy = document.createElement("object");
			
		proxy.type = object.type + proxyMimeSuffix;
		proxy.setAttribute("type", object.type + proxyMimeSuffix);
		
		proxy.setAttribute("firetv-data", data);
		if (data && data != "" && proxy.onDataChanged){
			proxy.onDataChanged("", data);
		}
		proxy.setAttribute("firetv-proxy-id", proxyId);
		
		var i, attr;
		for (i=0; i<object.attributes.length; i++){
			attr = object.attributes[i];
			if (attr.name != "type" && attr.name != "data" && attr.name != "src" && attr.name != "disabled"){
				proxy.setAttribute(attr.name, attr.value);
			}
		}
		
		object.removeAttribute("id");
		object.removeAttribute("class");
		object.setAttribute("firetv-proxy-id", proxyId);
		
		
		object.parentNode.insertBefore(proxy, object);
		object.parentNode.removeChild(object);
		return proxy;
	};
	
	var isProxifiedType = function(object){
		 return (proxifiedMimeTypes.indexOf(object.type) > -1);
	};
	
	var isProxy = function(object){
		 return object.type.match(/\+firetv$/);
	};
	
	var getProxy = function(object){
		var proxy = null;
		var selector;
		if (object.hasAttribute("firetv-proxy-id")) {
			selector = object.tagName.toLowerCase() + "[firetv-proxy-id='"
			+ object.getAttribute("firetv-proxy-id") + "']";
			proxy = document.querySelector(selector);
		} else {
			proxy = createProxy(object);
		}
		return proxy;
	};

	// -- HTMLObjectElement override
	var originalobjecttypesetter = (function() {
		return HTMLObjectElement.prototype.__lookupSetter__("type");
	})();
	
	var originalobjectdatasetter = (function() {
		return HTMLObjectElement.prototype.__lookupSetter__("data");
	})();
	
	var originalobjectdatagetter = (function() {
		return HTMLObjectElement.prototype.__lookupGetter__("data");
	})();
	
	var originalobjectstylegetter = (function() {
		return HTMLObjectElement.prototype.__lookupGetter__("style");
	})();
	
	var dataGetter = function(){
		var d = this.getAttribute("firetv-data")
		if (d) {
			return d;
		}
		return originalobjectdatagetter.apply(this, []);
	};

	HTMLObjectElement.prototype.__defineGetter__("data", dataGetter);
	HTMLObjectElement.prototype.__defineGetter__("src", dataGetter); // for replaced embeds
	
	
	var dataSetter = function(val){
		var proxy, previousVal;
		if(isProxifiedType(this) ){
			proxy = getProxy(this);
			previousVal = proxy.getAttribute("firetv-data");
			proxy.setAttribute("firetv-data", val);
			originalobjectdatasetter.apply(proxy, [""]);
			if (proxy.onDataChanged){
				proxy.onDataChanged(previousVal, val);
			}
		} else if(isProxy(this)){
			previousVal = this.getAttribute("firetv-data");
			this.setAttribute("firetv-data", val);
			if (this.onDataChanged){
				this.onDataChanged(previousVal, val);
			}
		} else {
			originalobjectdatasetter.apply(this, [val]);
		}
	};
	HTMLObjectElement.prototype.__defineSetter__("data", dataSetter);
	HTMLObjectElement.prototype.__defineSetter__("src", dataSetter); // for replaced embeds
	
	HTMLObjectElement.prototype.__defineSetter__("type", function(val){
		var proxy, previousVal;
		if(isProxy(this)){
			var originalType  = this.type.replace(proxyMimeSuffixRegexp, "");
			var newType = val + proxyMimeSuffix;
			if (newType != this.type){
				originalobjecttypesetter.apply(this, [newType]);
			}
		} else {
			originalobjecttypesetter.apply(this, [val]);
		}
	});
	
	HTMLObjectElement.prototype.__defineGetter__("style", function(){
		if(isProxifiedType(this)){
			return originalobjectstylegetter.apply(getProxy(this), []);;
		}
		return originalobjectstylegetter.apply(this, []);
	});
	

	
	
	var definePropertiesForTypes = function definePropertiesForTypes(initialPrototype, properties, types, needsProxy) {
		var newPrototype = properties;
		var key;
		var i;
		if (needsProxy) {
			for (i = 0; i < types.length; i++) {
				if (proxifiedMimeTypes.indexOf(types[i]) == -1) {
					proxifiedMimeTypes.push(types[i]);
				}
			}
		}
		for (key in newPrototype) {
			if (newPrototype.hasOwnProperty(key)) {
				for (i = 0; i < types.length; i++) {
					// -- clean
					(function() {
						var type = (needsProxy) ? typeToId(types[i] + proxyMimeSuffix) : typeToId(types[i]);
						var prop = key;
						delete initialPrototype["__firetv__" + type + "_val_" + prop];
						delete initialPrototype["__firetv__" + type + "_get_" + prop];
						delete initialPrototype["__firetv__" + type + "_set_" + prop];
						// -- ownval is type independant
						delete initialPrototype["__firetv__ownval_" + prop];
					})();
					// -- define per-type properties + generic get/set
					(function() {
						var prop = key;
						var origDesc = newPrototype[prop];

						var initialValue = origDesc.value;
						var initialWritable = (typeof origDesc.writable != "undefined") ? (!!origDesc.writable) : false;
						var initialGet = origDesc.get;
						var initialSet = origDesc.set;

						var type = (needsProxy) ? typeToId(types[i] + proxyMimeSuffix) : typeToId(types[i]);

						if (typeof initialValue != "undefined") {
							Object.defineProperty(initialPrototype, "__firetv__" + type + "_val_" + prop, {
								value : initialValue,
								writable : initialWritable,
								enumerable : false, 
								configurable : true
							});
						} else {
							// -- get/set
							if (initialGet) {
								Object.defineProperty(initialPrototype, "__firetv__" + type + "_get_" + prop, {
									value : initialGet,
									writable : true,
									enumerable : false, 
									configurable : true
								});
							}
							if (initialSet) {
								Object.defineProperty(initialPrototype, "__firetv__" + type + "_set_" + prop, {
									value : initialSet,
									writable : true,
									enumerable : false, 
									configurable : true
								});
							}
						}
						// -- define non-enumerable own property (type
						// independant)
						Object.defineProperty(initialPrototype, "__firetv__ownval_" + prop, {
							value : undefined,
							writable : true,
							enumerable : false,
							configurable : true
						});
						// -- define generic wrappers get/set if not already
						// done
						if (!initialPrototype.hasOwnProperty(prop)) {
							Object.defineProperty(initialPrototype, prop, {
								get : function() {
									if(isProxifiedType(this)){
										var proxy = getProxy(this);
										var res = proxy[prop];
										if(typeof res == "function"){
											res = res.bind(proxy);
										}
										return res;
									} else {
										var typeId = typeToId(this.type);
										if (initialPrototype.hasOwnProperty("__firetv__" + typeId + "_val_" + prop)) {
											return this["__firetv__" + typeId + "_val_" + prop];
										} else if (typeof this["__firetv__" + typeId + "_get_" + prop] == "function") {
											return this["__firetv__" + typeId + "_get_" + prop].apply(this, []);
										} else {
											// default case
											return this["__firetv__ownval_" + prop];
										}
									}
								},
								set : function(val) {
									if(isProxifiedType(this)){
										getProxy(this)[prop] = val;
									} else {
										var typeId = typeToId(this.type);
										if (initialPrototype.hasOwnProperty("__firetv__" + typeId + "_val_" + prop)) {
											this["__firetv__" + typeId + "_val_" + prop] = val;
										} else if (typeof this["__firetv__" + typeId + "_set_" + prop] == "function") {
											this["__firetv__" + typeId + "_set_" + prop].apply(this, [ val ]);
										} else {
											this["__firetv__ownval_" + prop] = val;
										}
									}
								},
								configurable : true,
								enumerable : true
							// no way to make difference with property
							// overlapping (prop1 for type1 is enumerable and
							// prop1 for type2 is not) : so force it enumerable=true ?
							// (at least for debug)
							});
						}
					})();
				}
			}
		}
	};
	
	var defineObjectPropertiesForTypes = function defineObjectPropertiesForTypes(properties, types, needsProxy) {
		definePropertiesForTypes(HTMLObjectElement.prototype, properties, types, needsProxy)
	};
	
	var onDOMContentLoaded = function(event){
		event.currentTarget.removeEventListener("DOMContentLoaded", onDOMContentLoaded, false);
		if(proxifiedMimeTypes.length == 0){
			return;
		}
		var selector = "";
		var i;
		for(i=0; i<proxifiedMimeTypes.length; i++){
			selector += "[type='"+proxifiedMimeTypes[i]+"']:not([firetv-proxy-id])";
			if (i<proxifiedMimeTypes.length-1){
				selector += ", ";
			}
		}
		var objectsToProxify = document.querySelectorAll(selector);
		for (i=0; i<objectsToProxify.length; i++){
			createProxy(objectsToProxify[i]);
		}
	};
	var onDOMNodeInserted = function(event){
		var o = event.originalTarget;
		if ((o instanceof HTMLObjectElement || o instanceof HTMLEmbedElement) && isProxifiedType(o)){
			createProxy(o);
		}
	};
	var onDOMNodeRemoved = function(event){
		var o = event.originalTarget;
		if (o instanceof HTMLObjectElement || o instanceof HTMLEmbedElement){
			if (o.hasAttribute("firetv-proxy-id") && (o.hasAttribute("firetv-data")||o.hasAttribute("firetv-src"))){
				// destroy proxy ?
			}
		}
	};

	window.addEventListener("DOMContentLoaded", onDOMContentLoaded, false);
	document.addEventListener("DOMNodeInserted", onDOMNodeInserted, false);
	document.addEventListener("DOMNodeRemoved", onDOMNodeRemoved, false);
	
	Object.defineProperty(FireTVPlugin, "defineObjectPropertiesForTypes", {
		value : defineObjectPropertiesForTypes,
		writable : false,
		enumerable : false, 
		configurable : true
	});
})();