/* See license.txt for terms of usage */

// -- Channel class
(function(){
	function Channel(ccid, onid, tsid, sid, name) {
	    this.onid = onid;
	    this.tsid = tsid;
	    this.sid  = sid;
	    this.ccid = ccid;
	    this.name = this.longname = this.description = name;
	};
	Object.defineProperties(Channel, {
		"TYPE_TV"   : { value: 0,  writable: false, enumerable: true },
	    "TYPE_RADIO": { value: 1,  writable: false, enumerable: true },
	    "TYPE_OTHER": { value: 2,  writable: false, enumerable: true },
	    "ID_ANALOG" : { value: 0,  writable: false, enumerable: true },
	    "ID_DVB_C"  : { value: 10, writable: false, enumerable: true },
	    "ID_DVB_S"  : { value: 11, writable: false, enumerable: true },
	    "ID_DVB_T"  : { value: 12, writable: false, enumerable: true },
	    "ID_DVB_SI_DIRECT" : { value: 13, writable: false, enumerable: true },
	    "ID_DVB_C2" : { value: 14, writable: false, enumerable: true },
	    "ID_DVB_S2" : { value: 15, writable: false, enumerable: true },
	    "ID_DVB_T2" : { value: 16, writable: false, enumerable: true },
	    "ID_ISDB_C" : { value: 20, writable: false, enumerable: true },
	    "ID_ISDB_S" : { value: 21, writable: false, enumerable: true },
	    "ID_ISDB_T" : { value: 22, writable: false, enumerable: true },
	    "ID_ATSC_T" : { value: 30, writable: false, enumerable: true },
	    "ID_IPTV_SDS" : { value: 40, writable: false, enumerable: true },
	    "ID_IPTV_URI" : { value: 41, writable: false, enumerable: true },
	    "channelType" : { value: 0, writable: false, enumerable: true }, // TYPE_TV
	    "idType" : { value: 12, writable: false, enumerable: true }, // ID_DVB_T
	    "favourite" : { value: false, writable: false, enumerable: true }
	    //, "toString" : { value : function() { return this.name; }}
	});
	window.Channel= Channel;
})();

// -- ChannelList class
(function(){
	function ChannelList() {
	    this._list= [];
	    var i, c;
	    for(i=0; i<FireTVPlugin.channelList.length; i++) {
	    	c= FireTVPlugin.channelList[i];
	    	this._list.push(new Channel(c.ccid, c.onid, c.tsid, c.sid, c.name));
	    }
	};

	Object.defineProperties(ChannelList.prototype, {
		"_list" : { value: [], enumerable: false },
		"length": { get: function() {
			return this._list.length;
		}, enumerable: false },
		"item" : { value : function(index) {
		    return this._list[index];
		}, enumerable: true},
		"getChannel" : { value : function(channelID) {
		    for(var i=0; i<this._list.length; i++) {
		    	if(this._list[i].ccid==channelID) {
		    		return this._list[i];
		    	}
		    }
		    return null;
		}, enumerable: true},
		"getChannelByTriplet" : { value : function(onid, tsid, sid) {
			for(var i=0; i<this._list.length; i++) {
		        if( this._list[i].sid==sid
		         && this._list[i].tsid==tsid
		         && this._list[i].onid==onid) {
		            return this._list[i];
		        }
		    }
		    return null;
		}, enumerable: true}
	});
	window.ChannelList= ChannelList;
})();

// -- Channel Config
(function() {
	function ChannelConfig(){};
	Object.defineProperties(ChannelConfig.prototype, {
		channelList : { value : new ChannelList(), enumerable: true },
		favouriteLists : { value : null, enumerable: true },
		currentFavouriteList : { value : "", enumerable: true }
	});
	window.ChannelConfig= new ChannelConfig();
})();


(function() {

	var PlayState = {
		UNREALIZED : 0,
		CONNECTING : 1,
		PRESENTING : 2,
		STOPPED : 3
	};

	var ErrorState = {
		CHANNEL_NOT_SUPPORTED_BY_TUNER : 0,
		NO_SIGNAL : 1,
		TUNER_LOCKED_BY_OTHER_OBJECT : 2,
		PARENTAL_LOCK_ON_CHANNEL : 3,
		ENCRYPTED_CHANNEL_WITHOUT_KEY_MODULE : 4,
		UNKNOWN_CHANNEL : 5,
		CHANNEL_SWITCH_INTERRUPTED : 6,
		CANNOT_CHANGED_BECAUSE_RECORDING : 7,
		CANNOT_RESOLVE_URI_OF_IP_CHANNEL : 8,
		INSUFFICIENT_BANDWIDTH : 9,
		NO_CHANNEL_LIST : 10,
		INSUFFICIENT_RESOURCES : 11,
		CHANNEL_NOT_FOUND : 12,
		UNIDENTIFIED_ERROR : 100,
	};

	var CssBorderTypes = ["Top", "Right", "Left", "Bottom"];
	var CssBorderProperties = ["Width", "Color", "Style"];
	
	var broadcastControlPrototype = {
		"_currentChannel" : {
			value : undefined,
			writable : true,
			enumerable : false
		},
		"_fullScreen" : {
			value : false,
			writable : true,
			enumerable : false
		},
		"_playStateInternal" : {
			value : PlayState.UNREALIZED,
			writable : true
		},
		"_playState" : {
			get : function() {
				return this._playStateInternal;
			},
			set : function(val) {
				if (val != this._playStateInternal) {
					this._playStateInternal = val;
					if(val === PlayState.PRESENTING && this._needsApplyFullScreen){
						this._applyFullScreen();
					}
					if (this.onPlayStateChange) {
						this.onPlayStateChange.apply(this, [this._playStateInternal]);
					}
				}
			}
		},
		"_volume" : {
			value : 50,
			writable : true,
			enumerable : false
		},
		
		"_cssBorderSavedValue" : {
			value : {},
			writable : true,
			enumerable : false
		},
		"_cssBorderSaveValue" : {
			value : function _cssBorderSaveValue(node) {
				this._cssBorderSavedValue = {};
				var i,j, prop, val;
				for(i = 0; i<CssBorderTypes.length; i++) {
					for(j = 0; j<CssBorderProperties.length; j++) {
						prop = "border" + CssBorderTypes[i] + CssBorderProperties[j];
						val = node.style[prop];
						if (val != ""){
							this._cssBorderSavedValue[prop] = val;
						}
					}
				}
			},
			writable : true,
			enumerable : false
		},
		"_cssBorderRestoreValue" : {
			value : function _cssBorderRestoreValue(node) {
				var i;
				for(i in this._cssBorderSavedValue) {
					if (this._cssBorderSavedValue.hasOwnProperty(i)){
						node.style[i] = this._cssBorderSavedValue[i];
					}
				}
			},
			writable : true,
			enumerable : false
		},
		"_applyFullScreen" : {
			value : function _applyFullScreen(){
				this.setAttribute("firetv-fullscreen", this._fullScreen);
				if (this._fullScreen === true) {
					var bounds = FireTVPlugin.getBounds(this);
					this.style.overflow = "visible";
					if (this.parentNode){
						this._cssBorderSaveValue(this.parentNode);
						this.parentNode.style.border = "none";
					}
					this.svg.setAttribute("style", "width:" + FireTVPlugin.profileDefinition.resolution.width
							+ "px; height:" + FireTVPlugin.profileDefinition.resolution.height
							+ "px; position:absolute; top:-"+Math.round(bounds.top)+"px; left:-"+Math.round(bounds.left)+"px;");
				} else {
					this.style.overflow = "visible";
					this.svg.setAttribute("style", "width:100%; height:100%");
					if (this.parentNode){
						this.parentNode.style.removeProperty("border");
						this._cssBorderRestoreValue(this.parentNode);
					}
				}
				this._needsApplyFullScreen = false;
			},
			writable : false,
			enumerable : false
		},
		"_needsApplyFullScreen" : {
			value : false,
			writable : true,
			enumerable : false
		},
		"fullScreen" : {
			get : function fullScreen() {
				return this._fullScreen;
			},
			enumerable : true
		},
		"onChannelChangeError" : {
			value : null,
			writable : true,
			enumerable : true
		},
		"onChannelChangeSucceeded" : {
			value : null,
			writable : true,
			enumerable : true
		},
		"playState" : {
			get : function playState() {
				return this._playState;
			},
			enumerable : true
		},
		"onPlayStateChange" : {
			value : null,
			writable : true,
			enumerable : true
		},
		
		"setFullScreen" : {
			value : function setFullScreen(fullScreen) {
				if (fullScreen != this._fullScreen) {
					this._fullScreen = fullScreen;
					this._needsApplyFullScreen = true;
					if(this.svg && this._playState === PlayState.PRESENTING) {
						this._applyFullScreen();
					}
					if (this.onFullScreenChange) {
						//this.onFullScreenChange.apply(this, []);
						setTimeout(this.onFullScreenChange.bind(this), 2000);
					}
				}
			},
			writable : false,
			enumerable : true
		},
		"currentChannel" : {
			get : function currentChannel() {
				if (typeof this._currentChannel === "undefined" ){
					// -- first call
					var currCcid = FireTVPlugin.getPref("tv-channel");
					this._currentChannel = ChannelConfig.channelList.getChannel(currCcid);
				}
				return this._currentChannel;
			},
			enumerable : true
		},
		
		"setChannel" : {
			value : function setChannel(channel, trickplay) {
				if (channel == null){
					this._currentChannel = null;
					if (this.onChannelChangeSucceeded){
						this.onChannelChangeSucceeded.apply (this, [null]);
					}
					this.release();
					return;
				}
				
				var knownChannel = ChannelConfig.channelList.getChannel(channel.ccid);
				if (!knownChannel){
					if (this.onChannelChangeError){
						this.onChannelChangeError.apply (this, [channel, ErrorState.CHANNEL_NOT_FOUND]);
					}
					return;
				}
				
				this._currentChannel = knownChannel;
				
				var currCcid = FireTVPlugin.getPref("tv-channel");
				if (channel.ccid != currCcid){
					FireTVPlugin.setPref("tv-channel", channel.ccid);
				}
				
				this.style.display = "block";
				this.style.removeProperty("backgroundColor");
				this.svg = document.createElement("object");
				this.svg.setAttribute("type", "image/svg+xml");
				this.svg.setAttribute("style", "width:100%; height:100%");
				this.svg.setAttribute("tabindex", "-1");
				this.svg.setAttribute("data", "tv:");
//				this.svg.style.backgroundColor = "#000000";
				this.svg.firebugIgnore = true;
				while(this.firstChild){
					this.removeChild(this.firstChild);
				}
				this.appendChild(this.svg);
				if(this._fullScreen){
					this._applyFullScreen();
				}
				this._playState = PlayState.CONNECTING;
				this._playState = PlayState.PRESENTING;
				if (this.onChannelChangeSucceeded){
					this.onChannelChangeSucceeded.apply (this, [channel]);
				}
			},
			writable : false,
			enumerable : true
		},
		"nextChannel" : {
			value : function nextChannel() {
				var currentChannel = this.currentChannel;
				var i, index;
				for (i=0; i<ChannelConfig.channelList.length;i++){
					if(ChannelConfig.channelList.item(i).ccid == currentChannel.ccid){
						index = (ChannelConfig.channelList.length + (i+1)) % ChannelConfig.channelList.length;
						break;
					}
				}
				var newChannel = ChannelConfig.channelList.item(index);
				this.setChannel(newChannel, true);
			},
			writable : false,
			enumerable : true
		},
		
		"prevChannel" : {
			value : function nextChannel() {
				var currentChannel = this.currentChannel;
				var i, index;
				for (i=0; i<ChannelConfig.channelList.length;i++){
					if(ChannelConfig.channelList.item(i).ccid == currentChannel.ccid){
						index = (ChannelConfig.channelList.length + (i-1)) % ChannelConfig.channelList.length;
						break;
					}
				}
				var newChannel = ChannelConfig.channelList.item(index);
				this.setChannel(newChannel, true);
			},
			writable : false,
			enumerable : true
		},
		
		"bindToCurrentChannel" : {
			value : function bindToCurrentChannel() {
				if (this.currentChannel){
					this.setChannel(this.currentChannel);
					return this.currentChannel;
				}
				throw "Unable to bind to currentChannel : application is in broadcast independant state";
			},
			writable : false,
			enumerable : true
		},

		"getChannelConfig" : {
			value : function getChannelConfig() {
				return ChannelConfig;
			},
			writable : false,
			enumerable : true
		},
		"createChannelObject" : {
			value : function createChannelObject(idType, onid, tsid, sid, _sourceID, _ipBroadcastID){
				if (idType !== Channel.ID_DVB_T){
					throw "ERROR createChannelObject : Unsupported idType : " + idType + " (must be " + Channel.ID_DVB_T + ")";
				}
				var knownChannel = ChannelConfig.channelList.getChannelByTriplet( onid, tsid, sid);
				if (knownChannel) {
					return knownChannel;
				}
				// -- best effort
				var channel = new Channel("ccid:" + new String(Math.random()).substring(2), onid, tsid, sid);
				return channel;
			},
			writable : false,
			enumerable : true
		},
		"release" : {
			value : function release() {
				this.style.display = "none";
				this._playState = PlayState.UNREALIZED;
			},
			writable : false,
			enumerable : true
		},
		"stop" : {
			value : function stop() {
				this._playState = PlayState.STOPPED;
//				this.style.backgroundColor = "#000000";
				if(this.svg){
					this.svg.style.visibility = "hidden";
				}
			},
			writable : false,
			enumerable : true
		},
		"setVolume" : {
			value : function setVolume(volume) {
				if (volume >= 0 && volume <= 100){
					this._volume = volume;
					return true;
				}
				return false;
			},
			writable : false,
			enumerable : true
		},
		"getVolume" : {
			value : function getVolume() {
				return this._volume;
			},
			writable : false,
			enumerable : true
		}
	};

	FireTVPlugin.defineObjectPropertiesForTypes(broadcastControlPrototype, [ "video/broadcast" ]);

})();

