/* See license.txt for terms of usage */

// HbbTV remote control keys constants definitions.
// In theory, we could just affect any value to these constants (as the 
// application should only use the constants as the norm says), but experiment
// proves that a lot of application use the hardcoded values instead.
// So affect them values as defined in CE-HTML norm (CEA-2014)

KeyEvent.prototype.VK_RED    = KeyEvent.VK_RED   = VK_RED    = 403; // "RED";
KeyEvent.prototype.VK_GREEN  = KeyEvent.VK_GREEN = VK_GREEN  = 404; // "GREEN";
KeyEvent.prototype.VK_YELLOW = KeyEvent.VK_YELLOW= VK_YELLOW = 405; // "YELLOW";
KeyEvent.prototype.VK_BLUE   = KeyEvent.VK_BLUE  = VK_BLUE   = 406; // "BLUE";

KeyEvent.prototype.VK_0 = KeyEvent.VK_0 = VK_0 = 48; // "0";
KeyEvent.prototype.VK_1 = KeyEvent.VK_1 = VK_1 = 49; // "1";
KeyEvent.prototype.VK_2 = KeyEvent.VK_2 = VK_2 = 50; // "2";
KeyEvent.prototype.VK_3 = KeyEvent.VK_3 = VK_3 = 51; // "3";
KeyEvent.prototype.VK_4 = KeyEvent.VK_4 = VK_4 = 52; // "4";
KeyEvent.prototype.VK_5 = KeyEvent.VK_5 = VK_5 = 53; // "5";
KeyEvent.prototype.VK_6 = KeyEvent.VK_6 = VK_6 = 54; // "6";
KeyEvent.prototype.VK_7 = KeyEvent.VK_7 = VK_7 = 55; // "7";
KeyEvent.prototype.VK_8 = KeyEvent.VK_8 = VK_8 = 56; // "8";
KeyEvent.prototype.VK_9 = KeyEvent.VK_9 = VK_9 = 57; // "9";

KeyEvent.prototype.VK_UP    = KeyEvent.VK_UP    = VK_UP    = 38; // "UP";
KeyEvent.prototype.VK_DOWN  = KeyEvent.VK_DOWN  = VK_DOWN  = 40; // "DOWN";
KeyEvent.prototype.VK_LEFT  = KeyEvent.VK_LEFT  = VK_LEFT  = 37; // "LEFT";
KeyEvent.prototype.VK_RIGHT = KeyEvent.VK_RIGHT = VK_RIGHT = 39;
KeyEvent.prototype.VK_BACK  = KeyEvent.VK_BACK  = VK_BACK  = 461;// "BACK";
KeyEvent.prototype.VK_ENTER = KeyEvent.VK_ENTER = VK_ENTER = 13; // "OK";

KeyEvent.prototype.VK_STOP       = KeyEvent.VK_STOP       = VK_STOP       = 413; // "STOP";
KeyEvent.prototype.VK_PLAY_PAUSE = KeyEvent.VK_PLAY_PAUSE = VK_PLAY_PAUSE = "PLAY_PAUSE";
KeyEvent.prototype.VK_PLAY       = KeyEvent.VK_PLAY       = VK_PLAY       = 415; // "PLAY";
KeyEvent.prototype.VK_PAUSE      = KeyEvent.VK_PAUSE      = VK_PAUSE      = 19; // "PAUSE";

KeyEvent.prototype.VK_FAST_FWD   = KeyEvent.VK_FAST_FWD   = VK_FAST_FWD   = 412; // "FORWARD";
KeyEvent.prototype.VK_REWIND     = KeyEvent.VK_REWIND     = VK_REWIND     = 417; // "BACKWARD";

/*** optional values, or unavailable to applications
KeyEvent.VK_NEXT = 425;//"NEXT";
KeyEvent.VK_PREV = 424;//"PREV";
KeyEvent.VK_HOME = "HOME";
KeyEvent.VK_MENU = "MENU";
KeyEvent.VK_GUIDE= "GUIDE";
KeyEvent.VK_TELETEXT     = "TELETEXT";
KeyEvent.VK_SUBTITLES    = "SUBTITLES";
KeyEvent.VK_CHANNEL_UP   = "CHANNEL_UP";
KeyEvent.VK_CHANNEL_DOWN = "CHANNEL_DOWN";
KeyEvent.VK_VOLUME_UP    = "VOLUME_UP";
KeyEvent.VK_VOLUME_DOWN  = "VOLUME_DOWN";
KeyEvent.VK_MUTE         = "MUTE";
***/

KeySet = {
	RED : 0x1,
	GREEN : 0x2,
	YELLOW : 0x4,
	BLUE : 0x8,
	NAVIGATION : 0x10,
	VCR : 0x20,
	SCROLL : 0x40,
	INFO : 0x80,
	NUMERIC : 0x100,
	ALPHA : 0x200,
	OTHER : 0x400,
	value : 0x3ff,
	setValue : function(value, otherKeys) {
		this.value = value;
		// otherKeys ignored in hbbtv
	},
	filter : function(e) {
		var forward = true;

		switch (e.keyCode) {
		case KeyEvent.VK_RED:
			forward = KeySet.value & KeySet.RED;
			break;
		case KeyEvent.VK_GREEN:
			forward = KeySet.value & KeySet.GREEN;
			break;
		case KeyEvent.VK_YELLOW:
			forward = KeySet.value & KeySet.YELLOW;
			break;
		case KeyEvent.VK_BLUE:
			forward = KeySet.value & KeySet.BLUE;
			break;

		case KeyEvent.VK_0:
		case KeyEvent.VK_1:
		case KeyEvent.VK_2:
		case KeyEvent.VK_3:
		case KeyEvent.VK_4:
		case KeyEvent.VK_5:
		case KeyEvent.VK_6:
		case KeyEvent.VK_7:
		case KeyEvent.VK_8:
		case KeyEvent.VK_9:
			forward = KeySet.value & KeySet.NUMERIC;
			break;

		case KeyEvent.VK_UP:
		case KeyEvent.VK_DOWN:
		case KeyEvent.VK_LEFT:
		case KeyEvent.VK_RIGHT:
		case KeyEvent.VK_BACK:
		case KeyEvent.VK_ENTER:
			forward = KeySet.value & KeySet.NAVIGATION;
			break;

		case KeyEvent.VK_STOP:
		case KeyEvent.VK_PLAY_PAUSE:
		case KeyEvent.VK_PLAY:
		case KeyEvent.VK_PAUSE:
		case KeyEvent.VK_FAST_FWD:
		case KeyEvent.VK_REWIND:
			forward = KeySet.value & KeySet.VCR;
			break;
		}
		if (!forward) {
			if (e.type == "keydown" && typeof window.console != "undefined" && typeof window.console.info == "function") {
				window.console.info("[KeySet] KeyEvent " + e.keyCode + " filtered");
			}
			e.stopPropagation();
		}
	}
};

window.addEventListener("keypress", KeySet.filter, true);
window.addEventListener("keydown", KeySet.filter, true);
